from django.conf.urls import include, url
from django.db import transaction
from django.views.generic.base import RedirectView

from . import views
from .api import urls as api_urls
from .conf import settings

# dictionary of all LEPRIKON_URL_* attributes of settings
LEPRIKON_URL = dict(
    (attr.lower()[len("LEPRIKON_URL_") :], getattr(settings, attr))
    for attr in dir(settings)
    if attr.startswith("LEPRIKON_URL_")
)


def d_url(pattern, name):
    return url(pattern.format(**LEPRIKON_URL), transaction.atomic(getattr(views, name)), name=name)


app_name = "leprikon"

urlpatterns = [
    url(r"^$", RedirectView.as_view(url="{summary}/".format(**LEPRIKON_URL), permanent=True), name="index"),
    d_url(r"^{summary}/$", "summary"),
    d_url(r"^{create_account}/$", "user_create"),
    d_url(r"^{user}/{agreement}/$", "user_agreement"),
    d_url(r"^{user}/{email}/$", "user_email"),
    d_url(r"^{user}/$", "user_update"),
    d_url(r"^{user}/{password}/$", "user_password"),
    d_url(r"^{login}/$", "user_login"),
    d_url(r"^{logout}/$", "user_logout"),
    d_url(r"^{password_reset}/$", "password_reset"),
    d_url(r"^{password_reset}/done/$", "password_reset_done"),
    d_url(
        r"^{password_reset}/(?P<uidb64>[0-9A-Za-z_\-]+)/(?P<token>[0-9A-Za-z]{{1,13}}-[0-9A-Za-z]{{1,20}})/$",
        "password_reset_confirm",
    ),
    d_url(r"^{password_reset}/complete/$", "password_reset_complete"),
    d_url(r"^{leader}/$", "leader_summary"),
    d_url(r"^{leader}/{journal}/(?P<pk>[0-9]+)/$", "journal"),
    d_url(r"^{leader}/{journal}/{add}/(?P<subject>[0-9]+)/$", "journal_create"),
    d_url(r"^{leader}/{journal}/(?P<pk>[0-9]+)/{edit}/$", "journal_update"),
    d_url(r"^{leader}/{journal}/(?P<pk>[0-9]+)/{delete}/$", "journal_delete"),
    d_url(r"^{leader}/{journal}/{journalentry}/{add}/(?P<journal>[0-9]+)/$", "journalentry_create"),
    d_url(r"^{leader}/{journal}/{journalentry}/(?P<pk>[0-9]+)/$", "journalentry_update"),
    d_url(r"^{leader}/{journal}/{journalentry}/(?P<pk>[0-9]+)/{delete}/$", "journalentry_delete"),
    d_url(r"^{leader}/{journal}/{alternating}/$", "alternating"),
    d_url(r"^{leader}/{timesheets}/$", "timesheet_list"),
    d_url(r"^{leader}/{timesheets}/(?P<pk>[0-9]+)/$", "timesheet_detail"),
    d_url(r"^{leader}/{timesheets}/(?P<pk>[0-9]+)/{submit}/$", "timesheet_submit"),
    d_url(r"^{leader}/{timesheets}/{entry}/{add}/$", "timesheetentry_create"),
    d_url(r"^{leader}/{timesheets}/{entry}/(?P<pk>[0-9]+)/$", "timesheetentry_update"),
    d_url(r"^{leader}/{timesheets}/{entry}/(?P<pk>[0-9]+)/{delete}/$", "timesheetentry_delete"),
    d_url(r"^{leader}/{timesheets}/{journalentry}/(?P<pk>[0-9]+)/$", "journalleaderentry_update"),
    d_url(r"^{leader}/{timesheets}/{journalentry}/(?P<pk>[0-9]+)/{delete}/$", "journalleaderentry_delete"),
    d_url(r"^{leader}/(?P<subject_type>[^/]+)/$", "subject_list_mine"),
    d_url(r"^{leader}/(?P<subject_type>[^/]+)/(?P<pk>[0-9]+)/{edit}/$", "subject_update"),
    d_url(r"^{leader}/(?P<subject_type>[^/]+)/(?P<pk>[0-9]+)/{journals}/$", "subject_journals"),
    d_url(r"^{leader}/(?P<subject_type>[^/]+)/(?P<pk>[0-9]+)/{registrations}/$", "subject_registrations"),
    d_url(r"^{registrations}/$", "registration_list"),
    d_url(
        r"^{registrations}/(?P<pk>[^/]+)/{payment_request}-(?P<variable_symbol>[0-9]+).pdf$",
        "registration_payment_request",
    ),
    d_url(r"^{registrations}/(?P<pk>[^/]+)/(?P<slug>[^.]+).pdf$", "registration_pdf"),
    d_url(r"^{registrations}/(?P<pk>[^/]+)/{cancel}/$", "registration_cancel"),
    d_url(r"^{registration_link}/(?P<registration_link>[^/]+)/$", "registration_link"),
    d_url(r"^{registration_link}/(?P<registration_link>[^/]+)/(?P<pk>[0-9]+)/$", "registration_link_form"),
    d_url(r"^{payments}/$", "payment_list"),
    d_url(r"^{received_payments}/(?P<pk>[^/]+)/(?P<slug>[^.]+).pdf$", "received_payment_pdf"),
    d_url(r"^{returned_payments}/(?P<pk>[^/]+)/(?P<slug>[^.]+).pdf$", "returned_payment_pdf"),
    d_url(r"^{refund_requests}/(?P<pk>[0-9]+)/{add}/$", "refund_request_create"),
    d_url(r"^{refund_requests}/(?P<pk>[0-9]+)/$", "refund_request_update"),
    d_url(r"^{refund_requests}/(?P<pk>[0-9]+)/{delete}/$", "refund_request_delete"),
    d_url(r"^{payment_transfer}/(?P<pk>[0-9]+)/{add}/$", "payment_transfer_create"),
    d_url(r"^{donations}/(?P<pk>[0-9]+)/{add}/$", "donation_create"),
    d_url(r"^{donations}/$", "donation_list"),
    d_url(r"^{donations}/(?P<pk>[^/]+)/(?P<slug>[^.]+).pdf$", "donation_pdf"),
    d_url(r"^{participants}/$", "participant_list"),
    d_url(r"^{participants}/{add}/$", "participant_create"),
    d_url(r"^{participants}/(?P<pk>[0-9]+)/$", "participant_update"),
    d_url(r"^{participants}/(?P<pk>[0-9]+)/{delete}/$", "participant_delete"),
    d_url(r"^{participants}/{parent}/{add}/$", "parent_create"),
    d_url(r"^{participants}/{parent}/(?P<pk>[0-9]+)/$", "parent_update"),
    d_url(r"^{participants}/{parent}/(?P<pk>[0-9]+)/{delete}/$", "parent_delete"),
    d_url(r"^{billing_info}/$", "billing_info_list"),
    d_url(r"^{billing_info}/{add}/$", "billing_info_create"),
    d_url(r"^{billing_info}/(?P<pk>[0-9]+)/$", "billing_info_update"),
    d_url(r"^{billing_info}/(?P<pk>[0-9]+)/{delete}/$", "billing_info_delete"),
    d_url(r"^{messages}/$", "message_list"),
    d_url(r"^{messages}/(?P<slug>[^.]+)/$", "message_detail"),
    d_url(r"^{leaders}/$", "leader_list"),
    d_url(r"^{school_year}/$", "school_year"),
    d_url(r"^{reports}/$", "report_list"),
    d_url(r"^{reports}/{courses}/{payments}/$", "report_course_payments"),
    d_url(r"^{reports}/{courses}/{payments_status}/$", "report_course_payments_status"),
    d_url(r"^{reports}/{courses}/{stats}/$", "report_course_stats"),
    d_url(r"^{reports}/{events}/{payments}/$", "report_event_payments"),
    d_url(r"^{reports}/{events}/{payments_status}/$", "report_event_payments_status"),
    d_url(r"^{reports}/{events}/{stats}/$", "report_event_stats"),
    d_url(r"^{reports}/{orderables}/{payments}/$", "report_orderable_payments"),
    d_url(r"^{reports}/{orderables}/{payments_status}/$", "report_orderable_payments_status"),
    d_url(r"^{reports}/{orderables}/{stats}/$", "report_orderable_stats"),
    d_url(r"^{reports}/{debtors}/$", "report_debtors"),
    d_url(r"^{terms_conditions}/$", "terms_conditions"),
    url(r"^api/", include(api_urls, "api")),
]



from django.utils.formats import date_format
from django.utils.safestring import mark_safe
from django.utils.timezone import localtime


def datetime_with_by(attr, short_description):
    def f(self, obj):
        d = getattr(obj, attr)
        if d:
            d_formated = date_format(localtime(d), "SHORT_DATETIME_FORMAT")
            by = getattr(obj, f"{attr}_by")
            return mark_safe(f'<span title="{by}">{d_formated}</span>') if by else d_formated
        return "-"

    f.__name__ = f"{attr}_with_by"
    f.admin_order_field = attr
    f.short_description = short_description
    return f



from django.utils.translation import ugettext_lazy as _


class SendMailAdminMixin:
    actions = ("send_mail",)

    def send_mail(self, request, queryset):
        for obj in queryset.all():
            obj.send_mail()
        self.message_user(request, _("Selected items were sent by e-mail."))

    send_mail.short_description = _("Send selected items by e-mail")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['leprikon',
 'leprikon.admin',
 'leprikon.api',
 'leprikon.cms_plugins',
 'leprikon.conf',
 'leprikon.forms',
 'leprikon.forms.reports',
 'leprikon.management',
 'leprikon.management.commands',
 'leprikon.migrations',
 'leprikon.models',
 'leprikon.search_indexes',
 'leprikon.site',
 'leprikon.templatetags',
 'leprikon.views',
 'leprikon.views.reports']

package_data = \
{'': ['*'],
 'leprikon': ['locale/cs/LC_MESSAGES/*',
              'locale/en/LC_MESSAGES/*',
              'static/*',
              'static/admin/css/*',
              'static/admin/js/admin/*',
              'static/leprikon/css/*',
              'static/leprikon/fonts/*',
              'static/leprikon/img/*',
              'static/leprikon/js/*',
              'templates/admin/*',
              'templates/admin/inc/*',
              'templates/admin/leprikon/*',
              'templates/admin/leprikon/coursediscount/*',
              'templates/admin/leprikon/donation/*',
              'templates/admin/leprikon/eventdiscount/*',
              'templates/admin/leprikon/message/*',
              'templates/admin/leprikon/messagerecipient/*',
              'templates/admin/leprikon/orderablediscount/*',
              'templates/admin/leprikon/subjectreceivedpayment/*',
              'templates/admin/leprikon/subjectreturnedpayment/*',
              'templates/auth/widgets/*',
              'templates/cms/toolbar/items/*',
              'templates/leprikon/*',
              'templates/leprikon/admin/*',
              'templates/leprikon/cms/*',
              'templates/leprikon/cms/course/*',
              'templates/leprikon/cms/course_list/*',
              'templates/leprikon/cms/event/*',
              'templates/leprikon/cms/event_list/*',
              'templates/leprikon/cms/leader/*',
              'templates/leprikon/cms/leader_list/*',
              'templates/leprikon/cms/orderable/*',
              'templates/leprikon/cms/orderable_list/*',
              'templates/leprikon/donation_pdf/*',
              'templates/leprikon/journal_journal_pdf/*',
              'templates/leprikon/received_payment_pdf/*',
              'templates/leprikon/received_payment_received/*',
              'templates/leprikon/registration_approved/*',
              'templates/leprikon/registration_canceled/*',
              'templates/leprikon/registration_payment_request/*',
              'templates/leprikon/registration_pdf/*',
              'templates/leprikon/registration_received/*',
              'templates/leprikon/registration_refund_offer/*',
              'templates/leprikon/registration_refused/*',
              'templates/leprikon/reports/*',
              'templates/leprikon/returned_payment_pdf/*',
              'templates/leprikon/returned_payment_received/*',
              'templates/leprikon/static/*',
              'templates/leprikon/widgets/*']}

install_requires = \
['Django<2',
 'PyICU>=2.6,<3.0',
 'PyPDF2>=1.26.0,<2.0.0',
 'Whoosh>=2.7.4,<3.0.0',
 'certbot-nginx>=1.12.0,<2.0.0',
 'certbot>=1.12.0,<2.0.0',
 'cmsplugin-filer>=1.1.3,<2.0.0',
 'django-bankreader<0.7',
 'django-cms<3.7',
 'django-countries>=7.0,<8.0',
 'django-cron>=0.5.1,<0.6.0',
 'django-excel>=0.0.10,<0.0.11',
 'django-filer<1.6',
 'django-ganalytics>=0.7.0,<0.8.0',
 'django-haystack<2.8',
 'django-localflavor<2',
 'django-mathfilters>=1.0.0,<2.0.0',
 'django-multiselectfield>=0.1.12,<0.2.0',
 'django-pays>=0.1.0,<0.2.0',
 'django-qr-code<1.3',
 'django-redis==4.11.0',
 'django-staticfiles-downloader<0.3',
 'django-template-admin>=1.1.1,<2.0.0',
 'django-user-unique-email>=0.1.1,<0.2.0',
 'django-verified-email-field>=1.7.0,<2.0.0',
 'djangocms-googlemap<1.5',
 'djangocms-link<2.7',
 'djangocms-snippet<2.4',
 'djangocms-style<2.4',
 'djangocms-text-ckeditor<4',
 'djangocms-video<2.4',
 'gunicorn>=20.1.0,<21.0.0',
 'html2rml>=0.3.0,<0.4.0',
 'ipython>=7.20.0,<8.0.0',
 'lxml>=4.6.2,<5.0.0',
 'mysqlclient<2.1',
 'psycopg2-binary>=2.8.6,<3.0.0',
 'pyexcel-xlsxw>=0.6.1,<0.7.0',
 'python-memcached>=1.59,<2.0',
 'requests>=2.25.1,<3.0.0',
 'schwifty>=2021.5.2,<2022.0.0',
 'sentry-sdk>=0.19.5,<0.20.0',
 'social-auth-app-django>=4.0.0,<5.0.0',
 'sqlparse>=0.4.1,<0.5.0',
 'trml2pdf>=0.6,<0.7']

entry_points = \
{'console_scripts': ['leprikon = leprikon.__main__:main']}

setup_kwargs = {
    'name': 'leprikon',
    'version': '2.33.0',
    'description': 'Django CMS based IS for education',
    'long_description': 'Leprikón\n========\n\nLeprikón is web information system for leisure centres and other educational organizations.\n\n`www.leprikon.cz <https://www.leprikon.cz/>`__\n\n`Docker image <https://hub.docker.com/r/leprikon/leprikon/>`__\n\n\nInstallation with pip\n---------------------\n\n.. code:: shell\n\n    # create and enter an empty directory of your choice\n    mkdir leprikon && cd leprikon\n\n    # create and activate virtual environment\n    virtualenv env\n    . env/bin/activate\n\n    # install leprikon with all the requirements\n    pip install leprikon\n\n    # create database\n    leprikon migrate\n\n    # create admin user\n    leprikon createsuperuser\n\n    # run development server\n    ./manage.py runserver\n',
    'author': 'Jakub Dorňák',
    'author_email': 'jakub.dornak@misli.cz',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://leprikon.cz/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from django import forms
from django.utils.translation import ugettext_lazy as _

from ...models.events import Event
from ..form import FormMixin


class EventPaymentsForm(FormMixin, forms.Form):
    date_start = forms.DateField(label=_("Start date"))
    date_end = forms.DateField(label=_("End date"))


class EventPaymentsStatusForm(FormMixin, forms.Form):
    date = forms.DateField(label=_("Status for date"))


class EventStatsForm(FormMixin, forms.Form):
    date = forms.DateField(label=_("Statistics for date"))
    events = forms.ModelMultipleChoiceField(
        queryset=Event.objects.all(),
        label=_("Events"),
    )
    paid_only = forms.BooleanField(label=_("Count only paid registrations"), required=False)
    unique_participants = forms.BooleanField(label=_("Count only unique participants"), required=False)

    def __init__(self, *args, **kwargs):
        school_year = kwargs.pop("school_year")
        super().__init__(*args, **kwargs)
        self.fields["events"].queryset = Event.objects.filter(school_year=school_year)



from datetime import date
from random import randrange

from bankreader.models import Account
from django import forms
from django.contrib import admin
from django.http import HttpResponse
from django.shortcuts import render
from django.utils.html import format_html
from django.utils.translation import ugettext_lazy as _

from ..models.leprikonsite import LeprikonSite
from ..models.refundrequest import RefundRequest
from ..models.utils import BankAccount
from ..utils import amount_color, ascii, currency
from .export import AdminExportMixin
from .filters import SchoolYearListFilter, SubjectListFilter, SubjectTypeListFilter
from .utils import datetime_with_by


@admin.register(RefundRequest)
class RefundRequestAdmin(AdminExportMixin, admin.ModelAdmin):
    actions = ("export_as_abo",)
    date_hierarchy = "requested"
    ordering = ("-requested",)
    list_display = (
        "id",
        "requested_with_by",
        "registration",
        "amount_html",
        "bank_account",
    )
    list_export = (
        "id",
        "requested",
        "requested_by",
        "registration",
        "registration__variable_symbol",
        "amount",
        "bank_account",
        "iban",
        "bic",
    )
    list_filter = (
        ("registration__subject__school_year", SchoolYearListFilter),
        "registration__subject__department",
        "registration__subject__organization",
        ("registration__subject__subject_type", SubjectTypeListFilter),
        ("registration__subject", SubjectListFilter),
    )
    search_fields = (
        "registration__variable_symbol",
        "registration__subject__name",
        "registration__participants__first_name",
        "registration__participants__last_name",
        "registration__participants__birth_num",
    )
    raw_id_fields = ("registration",)

    def iban(self, obj):
        return obj.bank_account.compact

    iban.short_description = _("IBAN")

    def bic(self, obj):
        return obj.bank_account.bic

    bic.short_description = _("BIC (SWIFT)")

    def amount_html(self, obj):
        return format_html(
            '<b style="color: {color}">{amount}</b>',
            color=amount_color(obj.amount),
            amount=currency(abs(obj.amount)),
        )

    amount_html.short_description = _("amount")
    amount_html.admin_order_field = "amount"
    amount_html.allow_tags = True

    requested_with_by = datetime_with_by("requested", _("requested time"))

    def save_model(self, request, obj, form, change):
        if not change:
            obj.requested_by = request.user
        super().save_model(request, obj, form, change)

    def export_as_abo(self, request, queryset):
        class AccountForm(forms.Form):
            account = forms.ModelChoiceField(label=Account._meta.verbose_name, queryset=Account.objects.all())
            constant_symbol = forms.IntegerField(label=_("Constant symbol"), max_value=9999, required=False)
            message = forms.CharField(label=_("Message for recipient"), max_length=35, required=False)

        if request.POST.get("post", "no") == "yes":
            form = AccountForm(request.POST)
            if form.is_valid():
                return self._export_as_abo(
                    queryset,
                    BankAccount(form.cleaned_data["account"].iban),
                    form.cleaned_data["constant_symbol"] or 0,
                    form.cleaned_data["message"] or "",
                )
        else:
            form = AccountForm()

        adminform = admin.helpers.AdminForm(
            form,
            [(None, {"fields": list(form.base_fields)})],
            {},
            None,
            model_admin=self,
        )

        return render(
            request,
            "leprikon/admin/action_form.html",
            dict(
                title=_("Export setup"),
                opts=self.model._meta,
                adminform=adminform,
                media=self.media + adminform.media,
                action="export_as_abo",
                action_checkbox_name=admin.helpers.ACTION_CHECKBOX_NAME,
                select_across=request.POST["select_across"],
                selected=request.POST.getlist(admin.helpers.ACTION_CHECKBOX_NAME),
            ),
        )

    def _export_as_abo(self, queryset, bank_account: BankAccount, constant_symbol: int, message: str):
        batch_id = randrange(1, 1000)

        refund_requests: list[RefundRequest] = list(queryset)
        today = f"{date.today():%d%m%y}"
        message = ascii(message)
        site_name = ascii(LeprikonSite.objects.get_current().name)
        sum_amount = sum(rr.amount for rr in refund_requests if rr.amount)

        response = HttpResponse(content_type="text/abo; charset=ascii")
        response[
            "Content-Disposition"
        ] = f'attachment; filename="{ascii(str(self.model._meta.verbose_name))} {today} {batch_id}.abo"'

        lines = []

        # write header

        # header, date, customer name, customer number, batch number range, unused
        lines.append(f"UHL1{today}{site_name[:20].ljust(20)}0000000000001999000000000000")
        lines.append(f"1 1501 {batch_id:03}000 {bank_account.bank_code}")
        lines.append(f"2 {bank_account.account_code} {sum_amount}00 {today}")

        # write records

        for refund_request in refund_requests:
            if refund_request.amount:
                lines.append(
                    " ".join(
                        [
                            refund_request.bank_account.account_code,
                            str(refund_request.amount * 100),
                            str(refund_request.registration.variable_symbol),
                            f"{refund_request.bank_account.bank_code}{constant_symbol:04}",
                            "0",  # specific symbol
                            f"AV:{site_name[:35]}|{message[:35]}",
                        ]
                    )
                )

        # write footer

        lines.append("3 +")
        lines.append("5 +")
        lines.append("")

        response.write("\r\n".join(lines).encode("ascii", errors="ignore"))

        return response

    export_as_abo.short_description = _("Export selected records as ABO")
